==> shoplazza-cli-0.1.8.tgz: package/lib/checkout/util.js <==
const AdmZip = require('adm-zip');
const { cwd, configFile } = require('./config');
const fs = require('fs');
const path = require('path');
const { consoleError, consoleBlue } = require('./console');
const inquirer = require('inquirer');

const walkSync = (currentDirPath, callback) => {
  fs.readdirSync(currentDirPath, { withFileTypes: true }).forEach(function (dirent) {
    var filePath = path.join(currentDirPath, dirent.name);
    if (dirent.isFile()) {
      callback(filePath, dirent);
    } else if (dirent.isDirectory()) {
      walkSync(filePath, callback);
    }
  });
};

function getExtensionPathById(extensionId) {
  return path.join(cwd, !cwd.includes('/extensions') ? 'extensions' : '', extensionId);
}

/**
 * 根据id获取当前extension信息
 * @param {*} extensionId
 * @returns
 */
function getExtensionInfo(extensionId) {
  let result = {};
  const currentPath = path.resolve(cwd);
  const isInExtensionDir = !!fs.readdirSync(currentPath).find((f) => f === configFile);
  // 已经在一个extension内部
  if (isInExtensionDir) {
    const extensionId = currentPath.split('/').pop();
    result = {
      path: currentPath,
      id: extensionId
    };
  } else if (extensionId) {
    const extensionPath = path.join(cwd, !cwd.includes('/extensions') ? 'extensions' : '', extensionId);
    if (!fs.existsSync(extensionPath)) {
      consoleError(`The extension '${extensionId}' does not exist.`);
      throw `The extension '${extensionId}' does not exist.`;
    }

    result = {
      id: extensionId,
      path: extensionPath
    };
  } else {
    let dirPath = path.resolve(cwd);

    if (!dirPath.includes('extensions')) {
      consoleError('Please specify Extension id through --id.');
      throw 'not_id';
    }
    const extensionId = dirPath.split('/').pop();
    result = {
      id: extensionId,
      path: dirPath
    };
  }
  result.distPath = getDistPath();
  if (fs.existsSync(result.distPath)) {
    result.distName = fs.readdirSync(result.distPath).find((f) => f.includes(result.id));
  }
  return result;
}

function getExtensionConfig(id) {
  const info = getExtensionInfo(id);
  const config = path.join(info.path, configFile);

  if (!fs.existsSync(config)) {
    consoleError(`Extension '${info.id}' not exist ${configFile}.`);
    throw 'not_exist_config_file';
  }
  return JSON.parse(fs.readFileSync(config, 'utf-8'));
}

function getRootPath() {
  return cwd;
}

function getExtensionsPath() {
  return path.join(getRootPath(), 'extensions');
}

function getDistPath() {
  return path.join(getRootPath(), 'dist');
}

function getProjectConfig() {
  return require(path.join(getRootPath(), 'extension.config.js'));
}

function copy(src, dest) {
  const stat = fs.statSync(src);
  if (stat.isDirectory()) {
    copyDir(src, dest);
  } else {
    fs.copyFileSync(src, dest);
  }
}

function copyDir(srcDir, destDir) {
  fs.mkdirSync(destDir, { recursive: true });
  for (const file of fs.readdirSync(srcDir)) {
    const srcFile = path.resolve(srcDir, file);
    const destFile = path.resolve(destDir, file);
    copy(srcFile, destFile);
  }
}

/**
 * 选择模式，返回所选extension信息
 * @param {Array} extensionList extension列表
 * @param {String} type 类型：push | deploy | undeploy | preview
 * @returns {Object} 所选extension信息
 */
async function useSelectExtensionMode(extensionList = [], type = 'push') {
  const choices = extensionList.map((item, index) => ({
    name: (type === 'push' && index === 0) ? item.name : `${item.name}(${item.extension_id})`,
    value: item.extension_id
  }))
  if (!choices.length) {
    consoleBlue(`No extensions available to ${type}.`);
    process.exit(1);
  }
  const { extensionId } = await inquirer
    .prompt([
      {
        type: 'list',
        name: 'extensionId',
        message: `Please select an extension to ${type}:`,
        prefix: '*',
        loop: false,
        choices,
      },
    ])
  return extensionList.find((item) => item.extension_id === extensionId);
}

/**
 * 重写指定extension的extension.json文件内容
 * @param {String} extensionName extension名称
 * @param {*} extensionConfig extension配置
 */
function setExtensionConfig(extensionName, extensionConfig) {
  const info = getExtensionInfo(extensionName);
  const configFilePath = path.join(info.path, configFile);
  if (!fs.existsSync(configFilePath)) {
    consoleError(`Extension '${extensionName}' does not exist in ${configFile}.`);
    throw 'Config file does not exist';
  } else {
    fs.writeFile(configFilePath, JSON.stringify(extensionConfig, null, 2), (err) => {
      if (err) {
        consoleError(`Error writing file to disk: ${err}`);
      }
    });
  }
}

/**
 * 判断是否为本地extension
 * @param {String} extensionInfo extension信息
 */
function checkLocalExtension(extensionInfo) {
  const extensionPath = path.resolve(cwd, './extensions', extensionInfo.name || extensionInfo.extension_id); // todo: 此处extension_id兼容已存在extension，可考虑去掉
  if (fs.existsSync(extensionPath)) {
    const extensionJsonPath = path.resolve(extensionPath, './extension.json');
    const localExtensionInfo = JSON.parse(fs.readFileSync(extensionJsonPath, 'utf8'));
    if (localExtensionInfo.extensionId !== extensionInfo.extension_id) {
      consoleError(`The extension ${extensionInfo.name}(${extensionInfo.extension_id}) does not exist in your local project.`);
      throw `The extension ${extensionInfo.name}(${extensionInfo.extension_id}) does not exist in your local project.`;
    }
  } else {
    consoleError(`The extension ${extensionInfo.name}(${extensionInfo.extension_id}) does not exist in your local project.`);
    throw `The extension ${extensionInfo.name}(${extensionInfo.extension_id}) does not exist in your local project.`;
  }
}

module.exports = {
  copy,
  getRootPath,
  getDistPath,
  getExtensionsPath,
  getProjectConfig,
  getExtensionInfo,
  getExtensionConfig,
  getExtensionPathById,
  getFileJson: async (path) => JSON.parse(await fs.readFileSync(path)),
  debug: async (...log) => {
    ~process.argv.indexOf('--debug') && console.log(`${log.join('  ')}`);
  },
  zip: (filePath, uuid) => {
    const zip = new AdmZip();
    zip.addLocalFolder(filePath, uuid);
    zip.writeZip(`${cwd}/${uuid}.zip`);
  },
  useSelectExtensionMode,
  setExtensionConfig,
  checkLocalExtension
};

==> shoplazza-cli-1.0.2.tgz: package/lib/checkout/util.js <==
const AdmZip = require('adm-zip');
const { cwd, configFile } = require('./config');
const fs = require('fs');
const path = require('path');
const { consoleError, consoleBlue } = require('./console');
const inquirer = require('inquirer');

const walkSync = (currentDirPath, callback) => {
  fs.readdirSync(currentDirPath, { withFileTypes: true }).forEach(function (dirent) {
    var filePath = path.join(currentDirPath, dirent.name);
    if (dirent.isFile()) {
      callback(filePath, dirent);
    } else if (dirent.isDirectory()) {
      walkSync(filePath, callback);
    }
  });
};

function getExtensionPathById(extensionId) {
  return path.join(cwd, !cwd.includes('/extensions') ? 'extensions' : '', extensionId);
}

/**
 * 根据id获取当前extension信息
 * @param {*} extensionId
 * @returns
 */
function getExtensionInfo(extensionId) {
  let result = {};
  const currentPath = path.resolve(cwd);
  const isInExtensionDir = !!fs.readdirSync(currentPath).find((f) => f === configFile);
  // 已经在一个extension内部
  if (isInExtensionDir) {
    const extensionId = currentPath.split('/').pop();
    result = {
      path: currentPath,
      id: extensionId
    };
  } else if (extensionId) {
    const extensionPath = path.join(cwd, !cwd.includes('/extensions') ? 'extensions' : '', extensionId);
    if (!fs.existsSync(extensionPath)) {
      throw `The extension '${extensionId}' does not exist in your local project.`;
    }

    result = {
      id: extensionId,
      path: extensionPath
    };
  } else {
    let dirPath = path.resolve(cwd);

    if (!dirPath.includes('extensions')) {
      consoleError('Please specify Extension id through --id.');
      throw 'not_id';
    }
    const extensionId = dirPath.split('/').pop();
    result = {
      id: extensionId,
      path: dirPath
    };
  }
  result.distPath = getDistPath();
  if (fs.existsSync(result.distPath)) {
    result.distName = fs.readdirSync(result.distPath).find((f) => f.includes(result.id));
  }
  return result;
}

function getExtensionConfig(id) {
  const info = getExtensionInfo(id);
  const config = path.join(info.path, configFile);

  if (!fs.existsSync(config)) {
    consoleError(`Extension '${info.id}' not exist ${configFile}.`);
    throw 'not_exist_config_file';
  }
  return JSON.parse(fs.readFileSync(config, 'utf-8'));
}

function getRootPath() {
  return cwd;
}

function getExtensionsPath() {
  return path.join(getRootPath(), 'extensions');
}

function getDistPath() {
  return path.join(getRootPath(), 'dist');
}

function getProjectConfig() {
  return require(path.join(getRootPath(), 'extension.config.js'));
}

function copy(src, dest) {
  const stat = fs.statSync(src);
  if (stat.isDirectory()) {
    copyDir(src, dest);
  } else {
    fs.copyFileSync(src, dest);
  }
}

function copyDir(srcDir, destDir) {
  fs.mkdirSync(destDir, { recursive: true });
  for (const file of fs.readdirSync(srcDir)) {
    const srcFile = path.resolve(srcDir, file);
    const destFile = path.resolve(destDir, file);
    copy(srcFile, destFile);
  }
}

/**
 * 选择模式，返回所选extension信息
 * @param {Array} extensionList extension列表
 * @param {String} type 类型：push | deploy | undeploy | preview
 * @returns {Object} 所选extension信息
 */
async function useSelectExtensionMode(extensionList = [], type = 'push') {
  const choices = extensionList.map((item, index) => ({
    name: (type === 'push' && index === 0) ? item.name : `${item.name || item.extension_id}(${item.extension_id})`,
    value: item.extension_id
  }))
  if (!choices.length) {
    consoleBlue(`No extensions available to ${type}.`);
    process.exit(1);
  }
  const { extensionId } = await inquirer
    .prompt([
      {
        type: 'list',
        name: 'extensionId',
        message: `Please select an extension to ${type}:`,
        prefix: '*',
        loop: false,
        choices,
      },
    ])
  return extensionList.find((item) => item.extension_id === extensionId);
}

/**
 * 重写指定extension的extension.json文件内容
 * @param {String} extensionName extension名称
 * @param {*} extensionConfig extension配置
 */
function setExtensionConfig(extensionName, extensionConfig) {
  const info = getExtensionInfo(extensionName);
  const configFilePath = path.join(info.path, configFile);
  if (!fs.existsSync(configFilePath)) {
    consoleError(`Extension '${extensionName}' does not exist in ${configFile}.`);
    throw 'Config file does not exist';
  } else {
    fs.writeFile(configFilePath, JSON.stringify(extensionConfig, null, 2), (err) => {
      if (err) {
        consoleError(`Error writing file to disk: ${err}`);
      }
    });
  }
}

/**
 * 判断是否为本地extension
 * @param {String} extensionInfo extension信息
 */
function checkLocalExtension(extensionInfo) {
  const extensionPath = path.resolve(cwd, './extensions', extensionInfo.name || extensionInfo.extension_id); // todo: 此处extension_id兼容已存在extension，可考虑去掉
  if (fs.existsSync(extensionPath)) {
    const extensionJsonPath = path.resolve(extensionPath, './extension.json');
    const localExtensionInfo = JSON.parse(fs.readFileSync(extensionJsonPath, 'utf8'));
    if (localExtensionInfo.extensionId !== extensionInfo.extension_id) {
      consoleError(`The extension ${extensionInfo.name}(${extensionInfo.extension_id}) does not exist in your local project.`);
      throw `The extension ${extensionInfo.name}(${extensionInfo.extension_id}) does not exist in your local project.`;
    }
  } else {
    consoleError(`The extension ${extensionInfo.name}(${extensionInfo.extension_id}) does not exist in your local project.`);
    throw `The extension ${extensionInfo.name}(${extensionInfo.extension_id}) does not exist in your local project.`;
  }
}

module.exports = {
  copy,
  getRootPath,
  getDistPath,
  getExtensionsPath,
  getProjectConfig,
  getExtensionInfo,
  getExtensionConfig,
  getExtensionPathById,
  getFileJson: async (path) => JSON.parse(await fs.readFileSync(path)),
  debug: async (...log) => {
    ~process.argv.indexOf('--debug') && console.log(`${log.join('  ')}`);
  },
  zip: (filePath, uuid) => {
    const zip = new AdmZip();
    zip.addLocalFolder(filePath, uuid);
    zip.writeZip(`${cwd}/${uuid}.zip`);
  },
  useSelectExtensionMode,
  setExtensionConfig,
  checkLocalExtension
};
